/*
 * Copyright 2009-2010 by The Regents of the University of California
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * you may obtain a copy of the License from
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.uci.ics.hyracks.storage.am.common.api;

import edu.uci.ics.hyracks.api.exceptions.HyracksDataException;
import edu.uci.ics.hyracks.dataflow.common.data.accessors.ITupleReference;
import edu.uci.ics.hyracks.storage.am.common.ophelpers.IndexOp;
import edu.uci.ics.hyracks.storage.am.common.ophelpers.IndexOpContext;

public interface ITreeIndex {
	// init:

	public void create(int indexFileId, ITreeIndexFrame leafFrame,
			ITreeIndexMetaDataFrame metaFrame) throws HyracksDataException;

	public void open(int indexFileId);
	
	public void close();

	// operations:

	public void insert(ITupleReference tuple, IndexOpContext ictx)
			throws HyracksDataException, TreeIndexException;

	public void update(ITupleReference tuple, IndexOpContext ictx)
			throws HyracksDataException, TreeIndexException;

	public void delete(ITupleReference tuple, IndexOpContext ictx)
			throws HyracksDataException, TreeIndexException;

	public IndexOpContext createOpContext(IndexOp op,
			ITreeIndexFrame leafFrame, ITreeIndexFrame interiorFrame,
			ITreeIndexMetaDataFrame metaFrame);

	// bulk loading:

	public IIndexBulkLoadContext beginBulkLoad(float fillFactor,
			ITreeIndexFrame leafFrame, ITreeIndexFrame interiorFrame,
			ITreeIndexMetaDataFrame metaFrame) throws HyracksDataException;

	public void bulkLoadAddTuple(IIndexBulkLoadContext ictx,
			ITupleReference tuple) throws HyracksDataException;

	public void endBulkLoad(IIndexBulkLoadContext ictx)
			throws HyracksDataException;

	// search:
	public void diskOrderScan(ITreeIndexCursor icursor,
			ITreeIndexFrame leafFrame, ITreeIndexMetaDataFrame metaFrame,
			IndexOpContext ictx) throws HyracksDataException;

	// utility:

	public IFreePageManager getFreePageManager();

	public int getRootPageId();

	public ITreeIndexFrameFactory getLeafFrameFactory();

	public ITreeIndexFrameFactory getInteriorFrameFactory();

	public int getFieldCount();

	public IndexType getIndexType();
}
